/* Class PCF */

/*
    NUT nutrition software 
    Copyright (C) 1996-2012 by Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "Nut.h"
#include "db.h"
#include <FL/Fl_Tooltip.H>

static int PCF_count;
static bool need_to_save_meal_db;

void clear_out_pcfs(void)
{                               
PCF_count = 0;
}

void save_meal_db(void)
{
Fl::remove_idle((Fl_Timeout_Handler)save_meal_db);
if (!need_to_save_meal_db) return;
if (Fl::event_buttons() == 0)
 {
 write_meal_db();
 need_to_save_meal_db = 0;
 redraw_whole_schmeer();
 }
}

void pcf_cb(Fl_Widget *which, void *who)
{
PCF *who_we_are = (PCF *) who;
Nut_LightButton *button = (Nut_LightButton *) which;
if (button->value() == 1) who_we_are->turnon_tunein(button);
else who_we_are->dropout();
}

PCF::PCF (int x, int y, int w, int h, Fl_Color widgetcolor, MealFood *mealfood) : Fl_Group (x, y, w, h)
{
mf = mealfood;
this->tooltip("Automatic Portion Control:  Specify a food as the Protein, Carb, Fat, Potassium (\"K\"), or Vitamin E food and its quantity is always adjusted to meet that DV for the entire meal.  If a portion-controlled food's quantity goes negative, there is an excess over the DV of that nutrient in the non-portion-controlled foods.");
p = new Nut_LightButton(x, y, w/5, h, "P");
p->labelfont(FL_BOLD);
p->color(fl_lighter(MEALCOLOR));
p->callback(pcf_cb,this);
p->clear_visible_focus();
c = new Nut_LightButton(x+w/5, y, w/5, h, "C");
c->labelfont(FL_BOLD);
c->color(fl_lighter(MEALCOLOR));
c->callback(pcf_cb,this);
c->clear_visible_focus();
f = new Nut_LightButton(x+2*w/5, y, w/5, h, "F");
f->labelfont(FL_BOLD);
f->color(fl_lighter(MEALCOLOR));
f->callback(pcf_cb,this);
f->clear_visible_focus();
k = new Nut_LightButton(x+3*w/5, y, w/5, h, "K");
k->labelfont(FL_BOLD);
k->color(fl_lighter(MEALCOLOR));
k->callback(pcf_cb,this);
k->clear_visible_focus();
e = new Nut_LightButton(x+4*w/5, y, w/5, h, "E");
e->labelfont(FL_BOLD);
e->color(fl_lighter(MEALCOLOR));
e->callback(pcf_cb,this);
e->clear_visible_focus();
this->end();
}

void PCF::turnon_tunein(Nut_LightButton *button)
{
MealFood *mealfood;
int nut = -1;
if (button != p) p->value(0); 
else nut = PROCNT;
if (button != c) c->value(0); 
else nut = CHO_NONFIB;
if (button != f) f->value(0); 
else nut = FAT;
if (button != k) k->value(0); 
else nut = K;
if (button != e) e->value(0); 
else nut = VITE;
if (options.abnuts[nut] != -1) pcftweak(nut,mf->mfmealptr);
else 
 {
 button->value(0);
 return;
 }
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) (parent()->parent()->child(i));
 if (mealfood != mf)
  {
  if (button == p) mealfood->pcf->p->value(0);
  else if (button == c) mealfood->pcf->c->value(0);
  else if (button == f) mealfood->pcf->f->value(0);
  else if (button == k) mealfood->pcf->k->value(0);
  else mealfood->pcf->e->value(0);
  if (mf->mffoodptr == mealfood->mffoodptr)
   {
   mealfood->pcf->p->value(0);
   mealfood->pcf->c->value(0);
   mealfood->pcf->f->value(0);
   mealfood->pcf->k->value(0);
   mealfood->pcf->e->value(0);
   }
  }
 }
PCF::PCF_computation();
}

void PCF::dropout(void)
{
pcfclear(mf->mfmealptr);
PCF_count = parent()->parent()->children();
PCF_computation();
}

void PCF::gram_change(int index, double newval)
{
PCF_computation();
}

void PCF::refresh_values(void)
{
}

void PCF::PCF_computation()
{
MealFood *mealfood;
if (PCF_count == 0)
 {
 need_to_save_meal_db = 1;
 Fl::add_idle((Fl_Timeout_Handler)save_meal_db);
 return;
 }
pcfrun();
PCF_count = parent()->parent()->children();
for (int i = 0; i < PCF_count; i++)
 {
 mealfood = (MealFood *) parent()->parent()->child(i);
 mealfood->non_recursive_gram_change();
 }
need_to_save_meal_db = 1;
Fl::add_idle((Fl_Timeout_Handler)save_meal_db);
update_analysis();
}
